/*
 * Copyright (c) 2021 Huawei Device Co., Ltd.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { AsyncCallback, Callback } from './basic';

/**
 * interface of window manager
 * @devices tv, phone, tablet, wearable, car
 * @since 6
 */
declare namespace window {
  /**
   * Obtains a Window instance
   * @devices tv, phone, tablet, wearable, car
   * @since 6
   */
  function getTopWindow(callback: AsyncCallback<Window>): void;
  /**
   * Obtains a Window instance
   * @devices tv, phone, tablet, wearable, car
   * @since 6
   */
  function getTopWindow(): Promise<Window>;
  /**
   * Creates a subwindow
   * @param id Window ID
   * @param type Window type
   * @permission ohos.permission.SYSTEM_FLOAT_WINDOW
   * @since 7
   */
  function create(id: string, type: WindowType, callback: AsyncCallback<Window>): void;
  /**
   * Creates a subwindow
   * @param id Window ID
   * @param type Window type
   * @permission ohos.permission.SYSTEM_FLOAT_WINDOW
   * @since 7
   */
  function create(id: string, type: WindowType): Promise<Window>;
  /**
   * Finds a subwindow
   * @param id Window ID
   * @since 7
   */
  function find(id: string, callback: AsyncCallback<Window>): void;
  /**
   * Finds a subwindow
   * @param id Window ID
   * @since 7
   */
  function find(id: string): Promise<Window>;

  /**
   * Describes the window type
   * @devices tv, phone, tablet, wearable, car,
   * @since 7
   */
  enum WindowType {
    /**
     * Application window
     */
    TYPE_APP,

    /**
     * System pop-up window
     */
    TYPE_SYSTEM_ALERT,
  }

  /**
   * Describes the type of the area where the window cannot be displayed
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  enum AvoidAreaType {
    /**
     * Default area of the system
     */
    TYPE_SYSTEM,
    /**
     * Notch
     */
    TYPE_CUTOUT,
    /**
     * area for system gesture
     */
    TYPE_SYSTEM_GESTURE,
  }

  /**
   * Describes a rectangle
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  interface Rect {
    left: number;
    top: number;
    width: number;
    height: number;
  }

  /**
   * Describes the area where the window cannot be displayed
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  interface AvoidArea {
    /**
     * Rectangle on the left of the screen
     */
    leftRect: Rect;

    /**
     * Rectangle at the top of the screen
     */
    topRect: Rect;

    /**
     * Rectangle on the right of the screen
     */
    rightRect: Rect;

    /**
     * Rectangle at the bottom of the screen
     */
    bottomRect: Rect;
  }

  /**
   * Describes the window size
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  interface Size {
    /**
     * Window width
     */
    width: number;

    /**
     * Window height
     */
    height: number;
  }

  /**
   * Describes the properties of the status bar and navigation bar
   * @devices tv, phone, tablet, wearable, car
   * @since 6
   */
  interface SystemBarProperties {
    /**
     * Color of the status bar. The color is a hexadecimal value, for example, #00FF00 or #FF00FF00
     * @since 6
     */
    statusBarColor?: string;

    /**
     * Whether any icon on the status bar is highlighted
     * @since 7
     */
    isStatusBarLightIcon?: boolean;

    /**
     * Color of the navigation bar. The color is a hexadecimal value, for example, #00FF00 or #FF00FF00
     * @since 6
     */
    navigationBarColor?: string;

    /**
     * Whether any icon on the navigation bar is highlighted
     * @since 7
     */
    isNavigationBarLightIcon?: boolean;
  }

  /**
   * Describes the window properties
   * @devices tv, phone, tablet, wearable, car
   * @since 6
   */
  interface WindowProperties {
    /**
     * Window size
     * @since 7
     */
    windowRect: Rect;

    /**
     * Window type
     * @since 7
     */
    type: WindowType;

    /**
     * Screen brightness. The value ranges from 0 to 1. The value 1 indicates the maximum brightness
     * @since 6
     */
    brightness: number;

    /**
     * Whether the window is transparent. The default value is false
     * @since 7
     */
    isTransparent: boolean;

    /**
     * Whether the window is displayed in full screen mode. The default value is false
     * @since 6
     */
    isFullScreen: boolean;

    /**
     * Whether the screen is always on. The default value is false
     * @since 6
     */
    isKeepScreenOn: boolean;

    /**
     * Dimness of the window that is not on top. The value ranges from 0 to 1. The value 1 indicates the maximum dimness
     * @since 7
     */
    dimBehindValue: number;

    /**
     * Whether the window layout is in full-screen mode (whether the window is immersive). The default value is false
     * @since 7
     */
    isLayoutFullScreen: boolean;

    /**
     * Whether the window can gain focus. The default value is true
     * @since 7
     */
    focusable: boolean;

    /**
     * Whether the window is touchable. The default value is true
     * @since 7
     */
    touchable: boolean;

    /**
     * Whether the window is in privacy mode. The default value is false
     * @since 7
     */
    isPrivacyMode: boolean;

    /**
     * Whether the window has rounded corners
     * @since 7
     */
    isRoundCorner: boolean;
  }

  /**
   * interface for window
   * @devices tv, phone, tablet, wearable, car
   * @since 6
   */
  interface Window {
    /**
     * Sets the brightness for this window
     * @param brightness Brightness to set, which ranges from 0 to 1. The value 1 indicates the brightest
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setBrightness(brightness:number, callback: AsyncCallback<void>): void;
    /**
     * Sets the brightness for this window
     * @param brightness Brightness to set, which ranges from 0 to 1. The value 1 indicates the brightest
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setBrightness(brightness:number): Promise<void>;

    /**
     * Sets the background color for this window
     * @param color Background color to set. The color is a hexadecimal value, for example, #00FF00 or #FF00FF00
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setBackgroundColor(color: string, callback: AsyncCallback<void>): void;
    /**
     * Sets the background color for this window
     * @param color Background color to set. The color is a hexadecimal value, for example, #00FF00 or #FF00FF00
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setBackgroundColor(color: string): Promise<void>;

    /**
     * set the window type
     * @param type window type
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setWindowType(type: WindowType, callback: AsyncCallback<void>): void;
    /**
     * set the window type
     * @param type window type
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setWindowType(type: WindowType): Promise<void>;

    /**
     * Sets whether this window is transparent
     * @param isTransparent Whether the window is transparent
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setTransparent(isTransparent: boolean, callback: AsyncCallback<void>): void;
    /**
     * Sets whether this window is transparent
     * @param isTransparent Whether the window is transparent
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setTransparent(isTransparent: boolean): Promise<void>;

    /**
     * Sets whether to enable the full-screen mode for this window
     * @param isFullScreen Whether to enable the full-screen mode
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setFullScreen(isFullScreen: boolean, callback: AsyncCallback<void>): void;
    /**
     * Sets whether to enable the full-screen mode for this window
     * @param isFullScreen Whether to enable the full-screen mode
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setFullScreen(isFullScreen: boolean): Promise<void>;

    /**
     * Sets whether to keep the screen always on
     * @param isKeepScreenOn Whether to keep the screen always on
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setKeepScreenOn(isKeepScreenOn: boolean, callback: AsyncCallback<void>): void;
    /**
     * Sets whether to keep the screen always on
     * @param isKeepScreenOn Whether to keep the screen always on
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setKeepScreenOn(isKeepScreenOn: boolean): Promise<void>;

    /**
     * Sets the dimness of the window that is not on top
     * @param dimBehindValue Dimness of the window to set.
     * The value ranges from 0 to 1. The value 1 indicates the dimmest
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setDimBehind(dimBehindValue: number, callback: AsyncCallback<void>): void;
    /**
     * Sets the dimness of the window that is not on top
     * @param dimBehindValue Dimness of the window to set.
     * The value ranges from 0 to 1. The value 1 indicates the dimmest
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setDimBehind(dimBehindValue: number): Promise<void>;

    /**
     * Sets whether the window layout is in full-screen mode
     * @param isLayoutFullScreen Whether the window layout is in full-screen mode
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setLayoutFullScreen(isLayoutFullScreen: boolean, callback: AsyncCallback<void>): void;
    /**
     * Sets whether the window layout is in full-screen mode
     * @param isLayoutFullScreen Whether the window layout is in full-screen mode
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setLayoutFullScreen(isLayoutFullScreen: boolean): Promise<void>;

    /**
     * Sets whether this window can gain focus
     * @param isFocusable Whether the window can gain focus
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setFocusable(focusable: boolean, callback: AsyncCallback<void>): void;
    /**
     * Sets whether this window can gain focus
     * @param isFocusable Whether the window can gain focus
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setFocusable(focusable: boolean): Promise<void>;

    /**
     * Sets whether this window is touchable
     * @param isTouchable Whether the window is touchable
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setTouchable(touchable: boolean, callback: AsyncCallback<void>): void;
    /**
     * Sets whether this window is touchable
     * @param isTouchable Whether the window is touchable
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setTouchable(touchable: boolean): Promise<void>;

    /**
     * Sets whether this window is in privacy mode
     * @param isPrivacyMode Whether the window is in privacy mode
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setPrivacyMode(isPrivacyMode: boolean, callback: AsyncCallback<void>): void;
    /**
     * Sets whether this window is in privacy mode
     * @param isPrivacyMode Whether the window is in privacy mode
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setPrivacyMode(isPrivacyMode: boolean): Promise<void>;

    /**
     * Sets whether to display the status bar and navigation bar in this window
     * @param names Whether to display the status bar and navigation bar. For example, to display the status bar and navigation bar, set this parameter to ["status", "navigation"]
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setSystemBarEnable(names: Array<'status' | 'navigation'>, callback: AsyncCallback<void>): void;
    /**
     * Sets whether to display the status bar and navigation bar in this window
     * @param names Whether to display the status bar and navigation bar. For example, to display the status bar and navigation bar, set this parameter to ["status", "navigation"]
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    setSystemBarEnable(names: Array<'status' | 'navigation'>): Promise<void>;

    /**
     * Sets the properties of the status bar and navigation bar in this window
     * @param systemBarProperties Properties of the status bar and navigation bar to set
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setSystemBarProperties(systemBarProperties: SystemBarProperties, callback: AsyncCallback<void>): void;
    /**
     * Sets the properties of the status bar and navigation bar in this window
     * @param systemBarProperties Properties of the status bar and navigation bar to set
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    setSystemBarProperties(systemBarProperties: SystemBarProperties): Promise<void>;

    /**
     * Obtains the properties of this window. This method uses an asynchronous callback to return the window properties
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    getProperties(callback: AsyncCallback<WindowProperties>): void;
    /**
     * Obtains the properties of this window. This method uses an asynchronous callback to return the window properties
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     */
    getProperties(): Promise<WindowProperties>;

    /**
     * Obtains the area where this window cannot be displayed, for example, the system bar area and notch area
     * @param type Type of the area. TYPE_SYSTEM indicates the default area of the system. TYPE_CUTOUT indicates the notch area
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    getAvoidArea(type: AvoidAreaType, callback: AsyncCallback<AvoidArea>): void;
    /**
     * Obtains the area where this window cannot be displayed, for example, the system bar area and notch area
     * @param type Type of the area. TYPE_SYSTEM indicates the default area of the system. TYPE_CUTOUT indicates the notch area
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    getAvoidArea(type: AvoidAreaType): Promise<AvoidArea>;

    /**
     * Moves this window
     * @param x Distance that the window moves along the x-axis. A positive value indicates that the window moves to the right
     * @param y Distance that the window moves along the y-axis. A positive value indicates that the window moves downwards
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    moveTo(x: number, y: number, callback: AsyncCallback<void>): void;
    /**
     * Moves this window
     * @param x Distance that the window moves along the x-axis. A positive value indicates that the window moves to the right
     * @param y Distance that the window moves along the y-axis. A positive value indicates that the window moves downwards
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    moveTo(x: number, y: number): Promise<void>;

    /**
     * Changes the size of this window
     * @param width New width of the window
     * @param height New height of the window
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    resetSize(width: number, height: number, callback: AsyncCallback<void>): void;
    /**
     * Changes the size of this window
     * @param width New width of the window
     * @param height New height of the window
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    resetSize(width: number, height: number): Promise<void>;

    /**
     * Loads content to the subwindow
     * @param path Path of the page to which the content will be loaded
     * @since 7
     */
    loadContent(path: string, callback: AsyncCallback<void>): void;
    /**
     * Loads content to the subwindow
     * @param path Path of the page to which the content will be loaded
     * @since 7
     */
    loadContent(path: string): Promise<void>;

    /**
     * Shows the subwindow
     * @since 7
     */
    show(callback: AsyncCallback<void>): void;
    /**
     * Shows the subwindow
     * @since 7
     */
    show(): Promise<void>;

    /**
     * Checks whether the subwindow is displayed
     * @since 7
     */
    isShowing(callback: AsyncCallback<boolean>): void;
    /**
     * Checks whether the subwindow is displayed
     * @since 7
     */
    isShowing(): Promise<boolean>;

    /**
     * Destroys the subwindow
     * @since 7
     */
    destroy(callback: AsyncCallback<void>): void;
    /**
     * Destroys the subwindow
     * @since 7
     */
    destroy(): Promise<void>;

    /**
     * Sets whether the area outside the subwindow is touchable
     * @param touchable Whether the area outside the subwindow is touchable. The value true means that such an area is touchable, and false means the opposite
     * @since 7
     */
    setOutsideTouchable(touchable: boolean, callback: AsyncCallback<void>): void;
    /**
     * Sets whether the area outside the subwindow is touchable
     * @param touchable Whether the area outside the subwindow is touchable. The value true means that such an area is touchable, and false means the opposite
     * @since 7
     */
    setOutsideTouchable(touchable: boolean): Promise<void>;

    /**
     * Enables listening for changes to the area where the window cannot be displayed
     * @param type Listening type. Set it to systemAvoidAreaChange, which indicates listening for changes to the area where the window cannot be displayed
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    on(type: 'systemAvoidAreaChange', callback: Callback<AvoidArea>): void;

    /**
     * Disables listening for changes to the area where the window cannot be displayed
     * @param type Listening type. Set it to systemAvoidAreaChange, which indicates listening for changes to the area where the window cannot be displayed.
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    off(type: 'systemAvoidAreaChange', callback?: Callback<AvoidArea>): void;

    /**
     * Enables listening for window size changes
     * @param type Listening type. Set it to windowSizeChange, which indicates listening for window size changes
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    on(type: 'windowSizeChange', callback: Callback<Size>): void;

    /**
     * Disables listening for window size changes
     * @param type Listening type. Set it to windowSizeChange, which indicates listening for window size changes.
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    off(type: 'windowSizeChange', callback?: Callback<Size>): void;

    /**
     * Enables listening for keyboard height changes
     * @param type Listening type. Set it to keyboardHeightChange, which indicates listening for keyboard height changes
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    on(type: 'keyboardHeightChange', callback: Callback<number>): void;

    /**
     * Disables listening for keyboard height changes
     * @param type Listening type. Set it to keyboardHeightChange, which indicates listening for keyboard height changes
     * @devices tv, phone, tablet, wearable, car
     * @since 7
     */
    off(type: 'keyboardHeightChange', callback?: Callback<number>): void;

    /**
     * Enables listening for keyboard height changes
     * @param type Listening type. Set it to keyboardHeightChange, which indicates listening for keyboard height changes
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     * @deprecated since 7
     */
    on(type: 'keyboardHeightChange', callback: AsyncCallback<number>): void;

    /**
     * Disables listening for keyboard height changes
     * @param type Listening type. Set it to keyboardHeightChange, which indicates listening for keyboard height changes.
     * @devices tv, phone, tablet, wearable, car
     * @since 6
     * @deprecated since 7
     */
    off(type: 'keyboardHeightChange', callback?: AsyncCallback<number>): void;
  }
}

export default window;