/*
* Copyright (C) 2021 Huawei Device Co., Ltd.
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

import { AsyncCallback } from './basic';

/**
 * Indicates the rectangle of the display region.
 *
 * @name Rect
 * @since 7
 * @sysCap AAFwk
 * @devics phone, tablet
 * @permission N/A
 */
export interface Rect {
  /**
   * Indicate left edge coordinate value(X-axis value).
   *
   * @default 0
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  left: number;

  /**
   * Indicates top edge coordinate value(Y-axis value).
   *
   * @default 0
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  top: number;

  /**
   * Indicates width of the rectangle
   *
   * @default 0
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  width: number;

  /**
   * Indicates height of the rectangle
   *
   * @default 0
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  height: number;
}

/**
 * Indicates translate or scale of the layer.
 * [scaleX, skewY, skewX, scaleY, transX, transY]
 *
 * @name Matrix
 * @since 7
 * @sysCap AAFwk
 * @devics phone, tablet
 * @permission N/A
 */
export type Matrix = [number, number, number, number, number, number]

/**
 * Indicates the layerInfo object of the app layer {@link CallbackData}.
 * <p>Requires Permission: {@code ohos.permission.CONTROL_TASK_SYNC_ANIMATOR}
 *
 * @name layerInfo
 * @since 7
 * @sysCap AAFwk
 * @devics phone, tablet
 * @permission ohos.permission.CONTROL_TASK_SYNC_ANIMATOR
 */
export interface layerInfo {

  /**
   * Indicates the id of the layer.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  surfaceIndex: number;

  /**
   * Indicates whether the layer is invoked or disabled.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  mode: number;

  /**
   * Indicates layer positionX.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  positionX: number;

  /**
   * Indicates layer positionY.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  positionY: number;

  /**
   * Indicates height of the layer on the z-axis.
   * Layers with higher values will be displayed above.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  zIndex: number;

  /**
   * Indicates task ID corresponding to the layer.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  taskId: number;
}

/**
 * Indicates the callbackData object to return to {@link TaskSyncAnimationOptions.onAnimationBegin}.
 * Include information about each layer involved in the animation,
 * which is used to control the animation layer.
 * <p>Requires Permission: {@code ohos.permission.CONTROL_TASK_SYNC_ANIMATOR}
 *
 * @name CallbackData
 * @since 7
 * @sysCap AAFwk
 * @devics phone, tablet
 * @permission ohos.permission.CONTROL_TASK_SYNC_ANIMATOR
 */
export interface CallbackData {
  /**
   * Indicates ID of this animation, which is passed in {@link finishTaskSyncAnimation} for ending the animation.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  taskSyncAnimationId: number;

  /**
   * Indicates information about each layer involved in the animation.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  layerArray: layerInfo[];
}

/**
 * Indicates the parameters controls transformation of the layer and make animation.
 * <p>Requires Permission: {@code ohos.permission.CONTROL_TASK_SYNC_ANIMATOR}
 *
 * @name TaskSyncAnimationParameters
 * @since 7
 * @sysCap AAFwk
 * @devics phone, tablet
 * @permission ohos.permission.CONTROL_TASK_SYNC_ANIMATOR
 */
export interface TaskSyncAnimationParameters {
  /**
   * Indicates id of the layer{@link layerInfo.surfaceIndex}.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  surfaceIndex: number;

  /**
   * Indicates opacity of the layer.Scope:[0,1].
   *
   * @default 1
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  opacity: number;

  /**
   * Indicates borderRadius of the windowCrop.
   * If windowCrop.width or windowCrop.height is 0,this attribute does not take effect.
   *
   * @default 0
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  borderRadius: number;

  /**
   * Indicates  height of the layer z-axis.
   * Layers with higher values will be displayed above.
   *
   * @default 0
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  zIndex: number;

  /**
   * Indicates whether to hide the layer.
   *
   * @default false
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  hide: Boolean;

  /**
   * Indicates rectangle of the display region
   *
   * @default new Rect(0,0,0,0)
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  windowCrop: Rect;

  /**
   * Indicates translate or scale of the layer.
   *
   * @default new Matrix(1,0,0,1,0,0)
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  matrix: Matrix;

  /**
   * Reserved parameter, which does not need to be set currently.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  frameCount: number;
}

/**
 * Indicates enable the animation with passed basic information when starting an ability.
 * <p>Requires Permission: {@code ohos.permission.CONTROL_TASK_SYNC_ANIMATOR}
 *
 * @name TaskSyncAnimationOptions
 * @since 7
 * @sysCap AAFwk
 * @devics phone, tablet
 * @permission ohos.permission.CONTROL_TASK_SYNC_ANIMATOR
 */
export interface TaskSyncAnimationOptions {
  /**
   * Indicates whether to enable the animation capability
   *
   * @default false
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  enable: Boolean;

  /**
   * Reserved parameter, which does not need to be set currently.
   *
   * @default 0
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  type: number;

  /**
   * Indicates duration of the animation.
   * Scope:[0,2000]. Unit: ms.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  duration: number;

  /**
   * Indicates delay duration of the status bar animation relative to the animation start.
   * Scope:[0,duration]. Unit: ms.
   *
   * @default -
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  stateBarDelay: number;

  /**
   * Called when the animation is starts.
   *
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   * @param callbackData Include information about each layer involved in the animation.
   */
  onAnimationBegin(callbackData: CallbackData);

  /**
   * Called when the animation is canceled.
   *
   * @devices phone, tablet
   * @since 7
   * @sysCap AAFwk
   */
  onAnimationCancel();
}
