/*
 * Copyright (c) 2021 Huawei Device Co., Ltd.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { AsyncCallback, Callback } from './basic';

/**
 * interface of display manager
 * @devices tv, phone, tablet, wearable, car
 * @since 7
 */
declare namespace display {
  /**
   * Obtains the default display object
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  function getDefaultDisplay(callback: AsyncCallback<Display>): void;
  /**
   * Obtains the default display object
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  function getDefaultDisplay(): Promise<Display>;

  /**
   * Obtains all the display objects
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  function getAllDisplay(callback: AsyncCallback<Array<Display>>): void;
  /**
   * Obtains all the display objects
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  function getAllDisplay(): Promise<Array<Display>>;

  /**
   * Enables listening.
   * @param type：Listening type.
   * The available values are as follows:
   * add: listening for whether a display is added
   * remove: listening for whether a display is removed
   * change: listening for whether a display is changed
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  function on(type: 'add'|'remove'|'change', callback: Callback<number>): void;

  /**
   * Disables listening.
   * @param type：Listening type.
   * The available values are as follows:
   * add: listening for whether a display is added
   * remove: listening for whether a display is removed
   * change: listening for whether a display is changed
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  function off(type: 'add'|'remove'|'change', callback?: Callback<number>): void;

  /**
   * the state of display
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  enum DisplayState {
    /**
     * Unknown.
     */
    STATE_UNKNOWN = 0,
    /**
     * The display is shut down.
     */
    STATE_OFF,
    /**
     * The display is powered on.
     */
    STATE_ON,
    /**
     * The display is in sleep mode.
     */
    STATE_DOZE,
    /**
     * The display is in sleep mode, and the CPU is suspended.
     */
    STATE_DOZE_SUSPEND,
    /**
     * The display is in VR mode.
     */
    STATE_VR,
    /**
     * The display is powered on, and the CPU is suspended.
     */
    STATE_ON_SUSPEND,
  }

  /**
   * Properties of display, it couldn't update automatically
   * @devices tv, phone, tablet, wearable, car
   * @since 7
   */
  interface Display {
    /**
     * ID of the display.
     */
    id: number;

    /**
     * Name of the display.
     */
    name: string;

    /**
     * Whether the display is alive.
     */
    alive: boolean;

    /**
     * State of the display.
     */
    state: DisplayState;

    /**
     * Refresh rate of the display.
     */
    refreshRate: number;

    /**
     * Screen rotation angle of the display.
     */
    rotation: number;

    /**
     * Width of the display, in pixels.
     */
    width: number;

    /**
     * Height of the display, in pixels.
     */
    height: number;

    /**
     * Screen density of the display, in DPI.
     */
    densityDPI: number;

    /**
     * Screen density of the display, in pixels.
     */
    densityPixels: number;

    /**
     * Scaling factor for fonts displayed on the display.
     */
    scaledDensity: number;

    /**
     * Exact physical dots per inch of the screen in the horizontal direction.
     */
    xDPI: number;

    /**
     * Exact physical dots per inch of the screen in the vertical direction.
     */
    yDPI: number;
  }
}

export default display;